#!/bin/bash  
###################################################################
# (c) Copyright 2016-2017 Hewlett Packard Enterprise Development LP.
# Version A.12.10.00
####################################################################

########################## 
# Source utility functions. 
########################### 
PATH=$SGSBIN:/bin:/sbin:/usr/bin:/usr/sbin:/usr/lbin:$PATH
export PATH=$PATH

. /etc/cmcluster.conf 

if [[ -z $SG_UTILS ]]; then 
	SG_UTILS=$SGCONF/scripts/mscripts/utils.sh 
fi 
if [[ -f ${SG_UTILS} ]]; then 
	. ${SG_UTILS} 
	if (( $? != 0 )) 
		then 
			echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}" 
			exit 1 
	fi 
else 
	echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}" 
	exit 1 
fi 

KVMMNP_UTILS=$SGROOT/kvmtoolkit/hakvmmnputils.sh 
if [[ -f ${KVMMNP_UTILS} ]]; then 
	. ${KVMMNP_UTILS} 
	if (( $? != 0 )) 
		then 
			echo "ERROR: Unable to source package utility functions file: ${KVMMNP_UTILS}" 
			exit 1 
	fi 
else 
	echo "ERROR: Unable to find package utility functions file: ${KVMMNP_UTILS}" 
	exit 1 
fi
################################################################### 
# 
# Get the environment for this package through utility function 
# sg_source_pkg_env(). 
# 
################################################################### 
sg_source_pkg_env $* 

################################################################## 
# 
#       VALIDATE FUNCTION 
# 
################################################################# 
function validate_function 
{ 
    typeset filename="/etc/libvirt/qemu/$myvm.xml"
    typeset image
    typeset dirname
    typeset -i found=0
    typeset mnp_status
	

    # Validate that the uuid if given by user
    if [[ -n "$KVM_VM_UUID" ]]; then
        vm_name=`virsh domname $myuuid 2>/dev/null`
        if [[ $? -ne 0 ]]; then
            sg_log 0 "ERROR: Unable to get the VM name corresponding to UUID $myuuid"
            exit 1
        fi
        
        if [[ "$vm_name" != "$myvm" ]]; then
            sg_log 0 "ERROR: The given UUID $myuuid does not correspond to the VM Name $myvm."
            exit 1
        fi
    fi

    # Validate that the VM is not already configured in the Cluster
    line=`$cmviewcl -v -f line -s config 2>/dev/null | grep "VM_NAME=$myvm$"`
    if [[ $? -eq 0 ]]; then
        pkg=`echo "$line" | awk -F "[:|=]" '($1 == "package") {print $2}'`
        if [[ "$pkg" != "$SG_PACKAGE_NAME" ]]; then
            sg_log 0 "ERROR: The VM $myvm is already packaged in the cluster"
            exit_code=1
            return
        fi
    fi

    # Verify that the Virtual machine configuration exists
    # on all the package nodes
    if [[ ! -f $filename ]]; then    
        sg_log 0 "ERROR: Unable to find VM configuration file $filename of $myvm on the node $localnode"
        exit_code=1
        return
    fi
    
    # Verify if the Virtual machine indeed exists
    virsh domstate $myuuid > /dev/null 2>&1
    if [[ $? -ne 0 ]]; then
        sg_log 0 "ERROR: The virtual machine $myvm is not configured on the node $localnode"        
        exit_code=1
        return
    fi
    
    # Verify that the Virtual machine image file is on a 
    # shared disk whose mount point is specified in the 
    # Package.
    image=`cat $filename | grep "source file"| awk -F "[']" '{print $2}'`
    if [[ -z "$image" ]]; then
        sg_log 0 "ERROR: Unable to get the image file location from the VM configuration file $filename"        
        exit_code=1
        return
    fi
    
    # Loop through the FS mounts to confirm the image location
    # is specified in the package.
    for dirname in ${SG_FS_DIRECTORY}
    do
        echo $image | grep "$dirname" > /dev/null 2>&1
        if [[ $? -eq 0 ]]; then
            found=1
        fi    
    done
    
    if [[ $found -eq 0 ]]; then
        sg_log 0 "ERROR: The mount point for the image $image of the VM $myvm not specified in the package"                
        exit_code=1
        return
    fi
    
    # Checks for  cmcheckconf -v command
    if [[ "$SG_PRIV_CLUSTER_VERIFICATION" = "y" ]]; then          
        # Check the KVM_RES multi-node Package and inform the
        # user if libvirt is not being monitored on any node
        mnp_status=$(cmviewcl -f line -p ${MNP_RES} 2>/dev/null | \
                grep "^node:$localnode|status=up") 
        if [[ -z "${mnp_status}" ]]; then
            sg_log 0 "WARNING: KVM toolkit is not monitoring libvirt on the node $localnode"                  
            return
        fi
    fi
                
    sg_log 0 "KVM Toolkit validation Successful" 
        
	return 
}

############################################################# 
# 
#       START FUNCTION 
# 
############################################################ 
function start_function
{    
    typeset result
    typeset -i sleep_time=10
    typeset -i counter=0
    typeset -i ret=0
        
    result=`virsh domstate $myuuid 2>/dev/null`        
    if [[ $result == "running" ]]; then
        # The VM is already running, so start the package successfully
        return $ret
    else 
        # If the VM is not running, start it.
        virsh start $myuuid >/dev/null 2>&1         
        
        # Wait for the VM to boot up.
        # Script timeout value is the maximum amount of time
        # that the package will wait for the VM to start.
        while [[ $counter -lt 3 ]]
        do  
            sleep $sleep_time             
            result=`virsh domstate $myuuid 2>/dev/null`            
            if [[ $result != "running" ]]; then
                sg_log 0 "The VM $myvm is not in running state"
                ret=1
                exit_code=1
                counter=`expr $counter + 1`                   
                sg_log 0 "The VM $myvm is still not up, retrying...Attempt $counter"                       
                virsh start $myuuid >/dev/null 2>&1  
            else
                ret=0
                sg_log 0 "The VM $myvm is in running state"
                exit_code=0                
                return $ret
            fi                    
        done        
    fi
    
    if [[  $ret -ne 0 ]]; then
        sg_log 0 "Failed to start the package with VM $myvm"
        sg_log 0 "The VM $myvm is in $result state"
    fi
    return $ret
} 

############################################################# 
# 
#       STOP FUNCTION 
# 
############################################################ 
function stop_function 
{
    typeset process
    typeset pid
    typeset result
    
    # If the VM is already shutoff, then exit
    check_libvirt
    if [[ $? -eq 0 ]]; then 
        result=`virsh domstate $myuuid 2>/dev/null`
    else        
        ps -ef | grep qemu | grep -i "$myuuid" >/dev/null 2>&1
        if [[ $? -eq 0 ]]; then
            result="running"
        fi
    fi
    
    if [[ $result == "shut off" ]]; then
        sg_log 0 "The VM $myvm is already shut off."        
        return 0
    fi
    
    check_libvirt
    if [[ $? -eq 0 ]]; then 
        virsh shutdown $myvm > /dev/null 2>&1   
        sleep 10;
        if [[ $result != "shut off" ]]; then
            virsh destroy $myvm > /dev/null 2>&1
            sleep 3;
            result=`virsh domstate $myuuid 2>/dev/null`
        fi
    else      
        process=`ps -ef | grep qemu | grep -i "$myuuid"`
        pid=`echo "$process" | awk -F " " '{print $2}'`
        kill -9 $pid
        sleep 3
        ps -ef | grep qemu | grep -i "$myuuid" >/dev/null 2>&1
        if [[ $? -eq 0 ]]; then
            result="running"
        else
            result="shut off"
        fi
    fi        

    if [[ $result != "shut off" ]]; then
        sg_log 0 "ERROR: Failed to stop the VM $myvm"
        return 1
    else
        return 0
    fi
} 

#######################################################################
#
# function get_monitor_service gets the name of KVM 
# monitoring service.
#
#######################################################################
function get_monitor_service
{
   if [[ ${#SG_SERVICE_NAME[@]} -ne 0 ]]; then
       for (( i=0; i<${#SG_SERVICE_NAME[@]}; i++ ))
       do
           FOUND=`echo "${SG_SERVICE_CMD[$i]}" | grep -w 'hakvm.mon'` > /dev/null 2>&1
           if [[ "$FOUND" != "" ]]
           then
               service_name=${SG_SERVICE_NAME[$i]}
               service_cmd=${SG_SERVICE_CMD[$i]}
               sg_log 5 "Found monitoring service ${service_name}" 
               return 
           fi 
       done
   fi
}

#############################
#
# VALIDATE RECONFIGURE RESTART FUNCTION
# Return
#        0 if service related package attribute is NOT modified.
#        1 if service related package attribute is modified.
#
#############################
function validate_reconfigure_restart
{
    if [[ $KVM_MONITOR_INTERVAL = $PREV_KVM_MONITOR_INTERVAL ]] &&
       [[ $SG_SCRIPT_LOG_FILE = $PREV_SG_SCRIPT_LOG_FILE ]] &&
       [[ ${SG_EMAIL_ID[*]} =  ${PREV_SG_EMAIL_ID[*]} ]]
    then
        return 0
    fi
    return 1
}

############################################################
#
#       RECONFIGURE STOP FUNCTION 
#
############################################################
function reconfigure_stop_function
{
    if sg_check_if_reconfig_has_failed
    then
        sg_log 999 "WARNING: Not attempting to stop the KVM Monitoring service $service_name for online package modification."
        return
    fi
    
    validate_reconfigure_restart
    if [[ $? -ne 0 ]]
    then
        # Find the monitoring Service name
        get_monitor_service
        
        if ! ${SGSBIN}/cmhaltserv ${service_name} ; then
           sg_log 999 "ERROR: Could not stop  the service $service_name for online package modification."
           sg_set_reconfig_failure
           exit_code=1
        else
           sg_log 999 "SUCCESS: Successfully stopped the service $service_name online package modification."
           if ! sg_check_if_reconfig_has_failed ; then
                sg_log 0 "SUCCESS: Successfully stopped the service $service_name for online package modification."
           fi
        fi
    fi
    return
}

#############################
#
# RECONFIGURE START FUNCTION
#
#############################
function reconfigure_start_function
{
    validate_reconfigure_restart
    if [[ $? -ne 0 ]]
    then
        get_monitor_service
        msg="cmrunserv: Unable to start service $service_name (it is already active)"
        out=`${SGSBIN}/cmrunserv ${service_name} "${service_cmd}" 2>&1`
        exit_code=$?
        if [[ $exit_code -ne 0 ]] && [[ "$msg" != "$out" ]] ;
        then
            sg_log 999 "ERROR: Failed to start the service $service_name for online package modification."
            sg_set_reconfig_failure
            return 1
        elif [[ $exit_code -eq 0 ]] ; then
            sg_log 999 "SUCCESS: Successfully started the service $service_name for online package modification."
            if ! sg_check_if_reconfig_has_failed ; then
                 sg_log 0 "SUCCESS: Succesfully started service $service_name for online package modification. "
            fi
        fi
    fi
    return
}

#mainstart
################################## 
# 
#Start of Routines 
# 
################################# 

# Global Variables
typeset -i exit_code=0
typeset service_name
typeset service_cmd
typeset cmviewcl="$SGSBIN/cmviewcl"
typeset MNP_RES
typeset myvm
typeset myuuid
typeset localnode=`hostname | cut -d "." -f 1`

myvm=$KVM_VM_NAME
if [[ -z "$KVM_VM_UUID" ]]; then
    check_libvirt
    if [[ $? -eq 0 ]]; then
        myuuid=`virsh domuuid $myvm 2>/dev/null`
    else
        myuuid=`ps -ef | grep qemu | grep "name $myvm" | awk -F "-uuid" '{print $2}' | cut -d " " -f 2`
    fi
else
    myuuid=$KVM_VM_UUID
fi

if [[ -z $myuuid ]]; then
    sg_log 0 "ERROR: Unable to find the UUID of the VM $myvm"
    sg_log 0 "Please ensure that the VM configuration file is existing on this node."
    exit 1
fi

VCLVIEW=`$cmviewcl -s config -vf line`
MNP_RES=`echo "$VCLVIEW" | awk -F "[=|:]" '(($1 == "package") \
        && ($3 == "module_name") && ($4 == "tkit/kvm/kvmmnp") \
        && ($5 == "module_name") && ($6 == "tkit/kvm/kvmmnp"))\
        {print $2}'`
echo $MNP_RES | grep " " >/dev/null 2>&1
if [[ $? -eq 0 ]]; then
    sg_log 0 "ERROR: There are more than one MNP packages configured to monitor libvirtd in the cluster"
    exit 1
fi        
        
case $1 in
    validate)
        validate_function
        ;;
    start)
        start_function
        ;;
    stop)
        stop_function
        ;;
    reconfigure_stop)
        reconfigure_stop_function
        ;;
    reconfigure_start)
        reconfigure_start_function
        ;;
    *)
        sg_log 0 "Usage: ${0} [ start | stop | validate | reconfigure_start | reconfigure_stop ]"
    ;;
esac
exit $exit_code
#mainend
