#!/bin/bash
#######################################
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP.
# Version A.12.00.50
#######################################

########################## 
# Source utility functions. 
########################### 
PATH=$SGSBIN:/bin:/sbin:/usr/bin:/usr/sbin:/usr/lbin:$PATH
export PATH=$PATH

. /etc/cmcluster.conf 

if [[ -z $SG_UTILS ]]; then 
	SG_UTILS=$SGCONF/scripts/mscripts/utils.sh 
fi 
if [[ -f ${SG_UTILS} ]]; then 
	. ${SG_UTILS} 
	if (( $? != 0 )) 
		then 
			echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}" 
			exit 1 
	fi 
else 
	echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}" 
	exit 1 
fi 

# Global variables
typeset -i use_systemd=0

#######################################################################
# Function: cleanitup
# Description: This function cleans up files after every exit caused
# by error
#######################################################################
function cleanitup
{
    rm -f $policy_file > /dev/null 2>&1
    rm -f $mod_file > /dev/null 2>&1
    rm -f $pp_file > /dev/null 2>&1
}

#######################################################################
#
# function is_systemd checks if systemctl command is to be used
# if yes, then use_systemd is set to 1.
#
#######################################################################
function is_systemd
{
    if [[ -f /etc/redhat-release ]]; then
        grep "Maipo" /etc/redhat-release >/dev/null 2>&1
        if [[ $? -eq 0 ]]; then
            use_systemd=1
        fi
    else
        sles=`cat /etc/os-release | grep "VERSION=" | awk -F "[=-]" '{print $2}' | tr -d '"'`
        if [[ $sles -eq 12 ]]; then
            use_systemd=1
        fi
    fi
}
    
#######################################################################
#
# function check_libvirt check status of libvirtd
# if it is not running, return 1
#
#######################################################################
function check_libvirt
{
    typeset -i ret=0

    is_systemd
    if [[ $use_systemd -eq 1 ]]; then
        systemctl status libvirtd.service | grep "Active: active" > /dev/null 2>&1
        ret=$?
    else
        service libvirtd status > /dev/null 2>&1
        ret=$?
    fi
    if [[ $ret -ne 0 ]]; then
        return 1
    fi
    count=0
    return $ret
}

#######################################################################
#
# function create_and_insert_policy attempts to create and
# insert the SELinux policy required by KVM MNP
#
#######################################################################
function create_and_insert_policy
{   
    typeset policy_file="/tmp/mnprespol.te"
    typeset mod_file="/tmp/mnprespol.mod"
    typeset pp_file="/tmp/mnprespol.pp"
    
    # Remove any pre-existing SG policy files
    cleanitup

    # Create the file    
    cat <<EOF > $policy_file
module mnprespol 1.0;

require {
        type inetd_child_t;
        type virtd_t;
        type virtd_exec_t;
        class process { transition };
        class file { read getattr execute open };
}

#============= inetd_child_t ==============
allow inetd_child_t virtd_t:process transition;
allow inetd_child_t virtd_exec_t:file { read getattr execute open };
type_transition inetd_child_t virtd_exec_t : process virtd_t;
EOF

    if [[ ! -f /usr/bin/checkmodule ]]; then
        sg_log 0 "Unable to build Custom SELinux policy as checkmodule binary is missing in the system"
        sg_log 0 "Please start libvirtd using service or systemctl command before attempting to"
        sg_log 0 "start the package again"
        return 1
    fi
    
    # Build it
    result=`checkmodule -M -m -o $mod_file $policy_file 2>&1`
    if [[ $? -ne 0 ]]; then
        sg_log 0 "Failed to build SELinux Policy module due to"
        sg_log 0 "$result"
        cleanitup
        return 1
    fi
    # Package it
    semodule_package -o $pp_file -m $mod_file >/dev/null 2>&1
    if [[ $? -ne 0 ]]; then
        sg_log 0 "Failed to Package SELinux Policy module"        
        cleanitup
        return 1
    fi
    # Insert it
    semodule -i $pp_file >/dev/null 2>&1
    if [[ $? -ne 0 ]]; then
        sg_log 0 "Failed to insert SELinux Policy module"        
        cleanitup
        return 1
    fi
    
    cleanitup
    return 0
}

#######################################################################
#
# function remove_policy attempts to remove the SELinux policy
# inserted by KVM MNP previously
#
#######################################################################
function remove_policy
{
    if [[ -f /usr/sbin/sestatus ]] && [[ $use_systemd -eq 0 ]]; then
        sestatus | grep "SELinux status:" | grep enabled >/dev/null 2>&1
        if [[ $? -eq 0 ]]; then
            sestatus | grep "Current mode:" | grep enforcing >/dev/null 2>&1
            if [[ $? -eq 0 ]]; then
                semodule -r mnprespol > /dev/null 2>&1
                if [[ $? -ne 0 ]]; then
                    sg_log 0 "Failed to remove SELinux Policy module mnprespol"
                    return 1
                fi
            fi
        fi
    fi
    return 0
}

#######################################################################
#
# function start_libvirt attempts to restart libvirtd
# if it is already not running
#
#######################################################################
function start_libvirt
{
    typeset -i ret=0

    is_systemd
    
    # If SELinux is enforced, add Custom Policies for SELinux
    # required for the libvirtd to start with the correct
    # security context.
    # After the libvirt is started successfully/failed, this policy
    # must be removed.
    if [[ -f /usr/sbin/sestatus ]] && [[ $use_systemd -eq 0 ]]; then
        sestatus | grep "SELinux status:" | grep enabled >/dev/null 2>&1
        if [[ $? -eq 0 ]]; then
            sestatus | grep "Current mode:" | grep enforcing >/dev/null 2>&1
            if [[ $? -eq 0 ]]; then
                create_and_insert_policy
                if [[ $? -ne 0 ]]; then
                    sg_log 0 "ERROR: Failed to create required SELinux policy"
                    return 1
                fi
            fi   
        fi
    fi
    
    if [[ $use_systemd -eq 1 ]]; then
        systemctl start libvirtd.service > /dev/null 2>&1
        ret=$?
    else
        service libvirtd start > /dev/null 2>&1
        ret=$?
    fi
    
    # wait for 3 seconds before removing the policy
    sleep 3
    remove_policy
    return $ret
}
